/*
 * n01_web
 *
 * n01_web.c
 *
 * Copyright (C) 1996-2009 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <winsock.h>
#include <process.h>

#include "..\n01_plugin.h"
#include "Memory.h"
#include "file.h"
#include "code.h"
#include "score_save.h"
#include "resource.h"

/* Define */
#define INI_FILE					TEXT("n01_web.ini")
#define LOG_FILE					TEXT("n01_web.log")
#define RESENT_PATH					TEXT("resent_web")

#define SEND_SIZE					4096	// Mobt@TCY
#define RECV_SIZE					4096	// Mobt@TCY

#define TIMEOUT						60		// ^CAEgb

#define WM_MAIN_RELOAD_FILE			(WM_APP + 101)
#define WM_MAIN_GET_CSV_PATH		(WM_APP + 102)
#define WM_MAIN_SOCK_ERROR			(WM_APP + 103)

#define LOG_LEVEL_DISABLE			0
#define LOG_LEVEL_STANDARD			1
#define LOG_LEVEL_ERROR				1
#define LOG_LEVEL_ALL				2

/* Global Variables */
HINSTANCE hInst;
TCHAR ini_path[MAX_PATH];
TCHAR log_path[MAX_PATH];
TCHAR resent_path[MAX_PATH];

int op_log_level;
int c_th = -1;
int enable;
TCHAR server[BUF_SIZE];
unsigned long serveraddr;
int port;
TCHAR url[BUF_SIZE];
int start_send;
int input_send;

TCHAR send_url[BUF_SIZE];
TCHAR proxy_server[BUF_SIZE];
int proxy_port;

typedef struct _THD_INFO {
	HWND hWnd;
	SOCKET soc;
} THD_INFO;

/* Local Function Prototypes */
static BOOL save_log(const TCHAR *action, const TCHAR *buf, const int level);
static BOOL save_error(const TCHAR *action, const int err_code);
static int get_url(char *URL, char *server, char *path);
static char *recv_data(const SOCKET soc, int *ret_len);
static BOOL response_check(const SOCKET soc);
static unsigned int __cdecl client_send_thread(void *ti);
static BOOL client_send_csv(void);

/*
 * DllMain - C
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
	TCHAR work_path[MAX_PATH];
	TCHAR *p, *r;
	WORD wVersionRequested;
	WSADATA wsaData;
	int nErrorStatus;
	char host_name[BUF_SIZE];

	switch (fdwReason) {
	case DLL_PROCESS_ATTACH:
		hInst = hInstance;
		// AvP[ṼpX擾
		GetModuleFileName(hInst, work_path, MAX_PATH - 1);
		for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
			if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
				p++;
				continue;
			}
#endif	// UNICODE
			if (*p == TEXT('\\') || *p == TEXT('/')) {
				r = p;
			}
		}
		*r = TEXT('\0');
		// pX쐬
		wsprintf(ini_path, TEXT("%s\\%s"), work_path, INI_FILE);
		wsprintf(log_path, TEXT("%s\\%s"), work_path, LOG_FILE);
		wsprintf(resent_path, TEXT("%s\\%s"), work_path, RESENT_PATH);

		// WinSock̏
		wVersionRequested = MAKEWORD(1, 1);
		nErrorStatus = WSAStartup(wVersionRequested, &wsaData);
		if (nErrorStatus != 0) {
			return FALSE;
		}
		gethostname(host_name, BUF_SIZE - 1);
		op_log_level = GetPrivateProfileInt(TEXT("option"), TEXT("log"), LOG_LEVEL_STANDARD, ini_path);
		enable = GetPrivateProfileInt(TEXT("web"), TEXT("enable"), 1, ini_path);
		GetPrivateProfileString(TEXT("web"), TEXT("url"), TEXT(""), send_url, BUF_SIZE - 1, ini_path);
		GetPrivateProfileString(TEXT("web"), TEXT("proxy_server"), TEXT(""), proxy_server, BUF_SIZE - 1, ini_path);
		proxy_port = GetPrivateProfileInt(TEXT("web"), TEXT("proxy_port"), 80, ini_path);
		start_send = GetPrivateProfileInt(TEXT("web"), TEXT("start_send"), 1, ini_path);
		input_send = GetPrivateProfileInt(TEXT("web"), TEXT("input_send"), 1, ini_path);

		if (op_log_level != 0) DeleteFile(log_path);

		if (*proxy_server == '\0') {
			port = get_url(send_url, server, url);
		} else {
			lstrcpy(url, send_url);
			lstrcpy(server, proxy_server);
			port = proxy_port;
		}
		break;

	case DLL_PROCESS_DETACH:
#ifdef _DEBUG
//		mem_debug();
#endif
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
	return TRUE;
}

/*
 * get_plugin_info - vOC擾
 */
__declspec(dllexport) BOOL CALLBACK get_plugin_info(const HWND hWnd, const int index, PLUGIN_GET_INFO *pgi)
{
	switch (index) {
	case 0:
		LoadString(hInst, IDS_STRING_TITLE, pgi->title, BUF_SIZE - 1);
		lstrcpy(pgi->func_name, TEXT("n01_web"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;
	}
	return FALSE;
}

/*
 * n01_web_initialize - vOC̏
 */
__declspec(dllexport) BOOL CALLBACK n01_web_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	return (CALLTYPE_MENU | CALLTYPE_START | CALLTYPE_SET_START | CALLTYPE_INPUT_START | CALLTYPE_INPUT_END | CALLTYPE_LEG_END);
}

/*
 * prop_proc - EBhEvV[W
 */
static BOOL CALLBACK prop_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	TCHAR buf[BUF_SIZE];

	switch (msg) {
	case WM_INITDIALOG:
		SendDlgItemMessage(hDlg, IDC_CHECK_ENABLE, BM_SETCHECK, enable, 0);
		SendDlgItemMessage(hDlg, IDC_EDIT_URL, WM_SETTEXT, 0, (LPARAM)send_url);
		SendDlgItemMessage(hDlg, IDC_EDIT_SERVER, WM_SETTEXT, 0, (LPARAM)proxy_server);
		SetDlgItemInt(hDlg, IDC_EDIT_PORT, proxy_port, FALSE);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, FALSE);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			enable = SendDlgItemMessage(hDlg, IDC_CHECK_ENABLE, BM_GETCHECK, 0, 0);
			SendDlgItemMessage(hDlg, IDC_EDIT_URL, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)send_url);
			SendDlgItemMessage(hDlg, IDC_EDIT_SERVER, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)proxy_server);
			proxy_port = GetDlgItemInt(hDlg, IDC_EDIT_PORT, NULL, FALSE);
			serveraddr = 0;
			if (*proxy_server == '\0') {
				port = get_url(send_url, server, url);
			} else {
				lstrcpy(url, send_url);
				lstrcpy(server, proxy_server);
				port = proxy_port;
			}

			wsprintf(buf, TEXT("%d"), op_log_level);
			WritePrivateProfileString(TEXT("option"), TEXT("log"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), enable);
			WritePrivateProfileString(TEXT("web"), TEXT("enable"), buf, ini_path);
			WritePrivateProfileString(TEXT("web"), TEXT("url"), send_url, ini_path);
			WritePrivateProfileString(TEXT("web"), TEXT("proxy_server"), proxy_server, ini_path);
			wsprintf(buf, TEXT("%d"), proxy_port);
			WritePrivateProfileString(TEXT("web"), TEXT("proxy_port"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), start_send);
			WritePrivateProfileString(TEXT("web"), TEXT("start_send"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), input_send);
			WritePrivateProfileString(TEXT("web"), TEXT("input_send"), buf, ini_path);
			EndDialog(hDlg, TRUE);
			break;

		case IDCANCEL:
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * n01_log_property - vOC̃vpeB
 */
__declspec(dllexport) BOOL CALLBACK n01_web_property(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_PROP), hWnd, prop_proc, (LPARAM)0);
	return PLUGIN_SUCCEED;
}

/*
 * n01_web
 */
__declspec(dllexport) int CALLBACK n01_web(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR csv_path[MAX_PATH];
	TCHAR tmp_path[MAX_PATH];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	TCHAR *p;

	switch (call_type) {
	case CALLTYPE_MENU:
		DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_PROP), hWnd, prop_proc, (LPARAM)0);
		break;

	case CALLTYPE_START:
		// CSVM
		if (enable == 0) {
			break;
		}
		// M̃t@C݂邩`FbN
		wsprintf(find_path, TEXT("%s\\*.csv"), resent_path);
		if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
			break;
		}
		do {
			if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
				continue;
			}
			// CSVt@CM
			client_send_csv();
			break;
		} while (FindNextFile(h_find_file, &find_data) == TRUE);
		FindClose(h_find_file);
		break;

	case CALLTYPE_SET_START:
		// JntO
		pei->lParam1 = 1;
		break;

	case CALLTYPE_INPUT_END:
		if (pei->lParam1 == 0) {
			break;
		}
		// JntȌ
		pei->lParam1 = 0;
		// JnԂ̍Đݒ
		GetLocalTime(&pei->si->start_time);
		if (start_send != 0) {
			n01_web(hWnd, CALLTYPE_LEG_END, pei, pei->si->current_leg);
		}
		break;

	case CALLTYPE_INPUT_START:
		if (pei->lParam1 != 0 || input_send == 0) {
			break;
		}
		param = pei->si->current_leg;
	case CALLTYPE_LEG_END:
		if (enable == 0) {
			break;
		}
		// CSV擾
		p = mem_alloc(sizeof(TCHAR) * (get_score_string_length(pei->si, param) + 1));
		if (p == NULL) {
			save_log(TEXT("CALLTYPE_LEG_END"), TEXT("mem_alloc Error"), LOG_LEVEL_ERROR);
			break;
		}
		get_score_string(pei->si, p, param);
		// t@C̐
		GetDateFormat(0, 0, &pei->si->start_time, TEXT("yyyyMMdd"), date, BUF_SIZE - 1);
		GetTimeFormat(0, 0, &pei->si->start_time, TEXT("HHmmss"), time, BUF_SIZE - 1);
		lstrcpy(csv_path, resent_path);
		lstrcat(csv_path, TEXT("\\"));
		wsprintf(csv_path + lstrlen(csv_path), TEXT("%d_%s_%s"), pei->si->current_set, date, time);
		lstrcpy(tmp_path, csv_path);
		lstrcat(tmp_path, TEXT(".tmp"));
		lstrcat(csv_path, TEXT(".csv"));
		//CSVۑ
		CreateDirectory(resent_path, NULL);
		if (file_write_buf(tmp_path, p, sizeof(TCHAR) * lstrlen(p)) == FALSE) {
			save_error(TEXT("file_write_buf"), GetLastError());
			mem_free(&p);
			break;
		}
		// CSV
		mem_free(&p);
		// t@C̈ړ
		if (CopyFile(tmp_path, csv_path, FALSE) == FALSE) {
			save_error(TEXT("CopyFile"), GetLastError());
			break;
		}
		DeleteFile(tmp_path);
		// CSVt@CM
		client_send_csv();
		break;
	}
	return PLUGIN_SUCCEED;
}

/*
 * save_log - O̕ۑ
 */
static BOOL save_log(const TCHAR *action, const TCHAR *buf, const int level)
{
	TCHAR time[BUF_SIZE];
	HANDLE hFile;
	DWORD ret;

	if (op_log_level < level) {
		return TRUE;
	}
	//ۑt@CJ
	hFile = CreateFile(log_path, GENERIC_WRITE, FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == NULL || hFile == (HANDLE)-1){
		return FALSE;
	}
	SetFilePointer(hFile, 0, NULL, FILE_END);

	GetTimeFormat(0, 0, NULL, NULL, time, BUF_SIZE - 1);
	lstrcat(time, TEXT(": "));

	WriteFile(hFile, time, sizeof(TCHAR) * lstrlen(time), &ret, NULL);
	WriteFile(hFile, action, sizeof(TCHAR) * lstrlen(action), &ret, NULL);
	if (buf != NULL) {
		WriteFile(hFile, TEXT(": "), sizeof(TCHAR) * lstrlen(TEXT(": ")), &ret, NULL);
		WriteFile(hFile, buf, sizeof(TCHAR) * lstrlen(buf), &ret, NULL);
	}
	WriteFile(hFile, TEXT("\r\n"), sizeof(TCHAR) * lstrlen(TEXT("\r\n")), &ret, NULL);
	CloseHandle(hFile);
	return TRUE;
}

/*
 * save_error - G[̕ۑ
 */
static BOOL save_error(const TCHAR *action, const int err_code)
{
	TCHAR err_str[BUF_SIZE];
	TCHAR buf[BUF_SIZE];
	TCHAR *p;

	*err_str = TEXT('\0');
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, err_code, 0, err_str, BUF_SIZE - 1, NULL);
	for (p = err_str; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n'); p++);
	*p = TEXT('\0');
	wsprintf(buf, TEXT("Error(%s)"), action);
	return save_log(buf, err_str, LOG_LEVEL_ERROR);
}

/*
 * get_url - URLT[oƃpXo
 */
static int get_url(TCHAR *URL, TCHAR *server, TCHAR *path)
{
	TCHAR *p, *r, *t, wk[8];		/* Ɨp̈ */
	int i, port = 80;

	if (strlen(URL) <= 7) {
		return -1;
	}
	/* XL[擾 */
	for (i = 0; i < 7; i++) {
		wk[i] = URL[i];
	}
	wk[i] = TEXT('\0');
	if (lstrcmpi(wk, TEXT("http://")) != 0) {
		return -1;
	}

	/* T[o擾 */
	for (p = URL + i, r = server; *p != TEXT('/') && *p != TEXT('\0'); p++, r++) {
		*r = *p;
	}
	*r = TEXT('\0');

	/* 擾y[W̃pX擾 */
	for (r = path; *p != TEXT('\0'); p++, r++) {
		*r = *p;
	}
	*r = TEXT('\0');

	/* T[o:|[g ƂȂĂꍇ͕ */
	for (p = server; *p != TEXT(':') && *p != TEXT('\0'); p++);
	if (*p == TEXT(':')) {
		for (t = p + 1, r = wk; *t != TEXT('\0'); t++, r++) {
			*r = *t;
		}
		*r = TEXT('\0');
		port = _ttoi(wk);
		*p = TEXT('\0');
	}

	/* y[W̎w肪ꍇ́A/ ([g)ݒ肷 */
	if (*path == TEXT('\0')) {
		strcpy(path, TEXT("/"));
	}
	return port;
}

/*
 * recv_data - f[^M
 */
static char *recv_data(const SOCKET soc, int *ret_len)
{
	char recv_buf[RECV_SIZE];
	char *ret_buf = NULL;
	char *tmp_buf;
	char *p = NULL;
	int recv_len;
	BOOL head_end = FALSE;

	*recv_buf = '\0';
	*ret_len = 0;
    do {
		recv_len = recv(soc, recv_buf, RECV_SIZE, 0);
		if (recv_len == SOCKET_ERROR) {
			if (recv_len == SOCKET_ERROR) save_error(TEXT("recv"), WSAGetLastError());
			mem_free(&ret_buf);
			return NULL;
		}
		if (recv_len == 0) {
			save_log(TEXT("R"), TEXT("recv_data() Connection Closed"), LOG_LEVEL_ERROR);
			break;
		}
		tmp_buf = mem_alloc(sizeof(char) * (*ret_len + recv_len + 1));
		if (tmp_buf == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			mem_free(&ret_buf);
			return NULL;
		}
		if (ret_buf != NULL) {
			CopyMemory(tmp_buf, ret_buf, *ret_len);
		}
		CopyMemory(tmp_buf + *ret_len, recv_buf, recv_len);
		mem_free(&ret_buf);
		ret_buf = tmp_buf;
		*ret_len += recv_len;
		*(ret_buf + *ret_len) = '\0';

		for (p = ret_buf; *p != '\0'; p++) {
			if (p + 6 < ret_buf + *ret_len && *p == '\r' &&
				*(p + 1) == '\n' &&
				*(p + 2) == '0' &&
				*(p + 3) == '\r' &&
				*(p + 4) == '\n' &&
				*(p + 5) == '\r' &&
				*(p + 6) == '\n') {
				break;
			}
		}
		if (*p != '\0') {
			break;
		}
	} while (1);

    return ret_buf;
}

/*
 * response_check - X|X̃`FbN
 */
static BOOL response_check(const SOCKET soc)
{
#define OK_LEN						3		// "+OK" ̃oCg
	char *buf, *p;
	int len;
	BOOL ret;

	if ((buf = recv_data(soc, &len)) == NULL) {
		return FALSE;
	}

	for(p = buf; *p != ' ' && *p != '\0'; p++);
	for(; *p == ' '; p++);
	
	ret = ((strlen(p) < OK_LEN || *p != '2' || *(p + 1) != '0' || *(p + 2) != '0') ? FALSE : TRUE);
	if (ret == FALSE) {
		save_log(TEXT("R"), buf, LOG_LEVEL_ERROR);
	} else {
		save_log(TEXT("R"), buf, LOG_LEVEL_ALL);
	}
	mem_free(&buf);
	return ret;
}

/*
 * URL_encode - URL encoding GR[h (RFC 2396)
 */
static const char cHex[] = "0123456789ABCDEF";
void URL_encode(unsigned char *buf, char *ret)
{
	unsigned char *p;
	char *r;

	for (p = buf, r = ret; *p != '\0'; p++) {
		if ((*p >= 'A' && *p <= 'Z') ||
			(*p >= 'a' && *p <= 'z') ||
			(*p >= '0' && *p <= '9')) {
			*(r++) = *p;
		} else {
			*(r++) = '%';
			*(r++) = cHex[*p >> 4];
			*(r++) = cHex[*p & 0xF];
		}
	}
	*r = '\0';
}

/*
 * client_send_thread - CSVM
 */
static unsigned int __cdecl client_send_thread(void *ti)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR path[MAX_PATH];
	SOCKET soc;
	struct sockaddr_in serversockaddr;
	char head_buf[SEND_SIZE];
	char set_buf[BUF_SIZE];
	char *url_str;
	char *file_buf;
	DWORD file_size;

	save_log(TEXT("Start Send Thread"), NULL, LOG_LEVEL_STANDARD);
	// \Pbg쐬
	save_log(TEXT("socket"), NULL, LOG_LEVEL_ALL);
	soc = socket(PF_INET, SOCK_STREAM, 0);
	if (soc == INVALID_SOCKET) {
		save_error(TEXT("socket"), WSAGetLastError());
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// ڑ
	save_log(TEXT("connect"), NULL, LOG_LEVEL_ALL);
	serversockaddr.sin_family = AF_INET;
	serversockaddr.sin_addr.s_addr = serveraddr;
	serversockaddr.sin_port = htons((unsigned short)port);
	ZeroMemory(serversockaddr.sin_zero, sizeof(serversockaddr.sin_zero));
	if (connect(soc, (struct sockaddr *)&serversockaddr, sizeof(serversockaddr)) == SOCKET_ERROR) {
		save_error(TEXT("connect"), WSAGetLastError());
		closesocket(soc);
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// t@C
	wsprintf(find_path, TEXT("%s\\*.csv"), resent_path);
	save_log(TEXT("Find File"), find_path, LOG_LEVEL_ALL);
	if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
		save_error(TEXT("FindFirstFile"), GetLastError());
		shutdown(soc, 2);
		closesocket(soc);
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	do {
		if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			continue;
		}
		// t@Cǂݍ
		wsprintf(path, TEXT("%s\\%s"), resent_path, find_data.cFileName);
		if ((file_buf = file_read_buf(path, &file_size)) == NULL) {
			save_error(TEXT("file_read_buf"), GetLastError());
			break;
		}
		// URLGR[h
		url_str = (char *)mem_alloc(file_size * 3 + 1);
		if (url_str == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			mem_free(&file_buf);
			break;
		}
		URL_encode(file_buf, url_str);
		mem_free(&file_buf);

		wsprintf(set_buf, "set=%d", _ttol(find_data.cFileName));
		// wb_̑M
		wsprintf(head_buf,"POST %s HTTP/1.1\r\nHost: %s:%d\r\nUser-Agent: n01_web/0.0\r\nContent-Length: %d\r\nConnection: Keep-Alive\r\n\r\n%s&csv=",
			url, server, port, strlen(set_buf) + 1 + strlen(url_str) + 4, set_buf);
		save_log(TEXT("S"), head_buf, LOG_LEVEL_ALL);
		if (send(soc, head_buf, strlen(head_buf), 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			mem_free(&url_str);
			break;
		}
		// Body̑M
		save_log(TEXT("S"), url_str, LOG_LEVEL_ALL);
		if (send(soc, url_str, strlen(url_str), 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			mem_free(&url_str);
			break;
		}
		mem_free(&url_str);
		if (response_check(soc) == TRUE) {
			if (file_size == file_get_size(path)) {
				// t@C폜
				save_log(TEXT("Delete"), path, LOG_LEVEL_ALL);
				if (DeleteFile(path) == FALSE) {
					save_error(TEXT("DeleteFile"), GetLastError());
				}
			}
		} else {
			break;
		}
	} while (FindNextFile(h_find_file, &find_data) == TRUE);
	FindClose(h_find_file);

	save_log(TEXT("closesocket"), NULL, LOG_LEVEL_ALL);
	shutdown(soc, 2);
	closesocket(soc);
	c_th = -1;
	save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
	_endthread();
	return 0;
}

/*
 * client_send_csv - CSVM
 */
static BOOL client_send_csv(void)
{
	struct hostent *serverhostent;

	if(*server == TEXT('\0')) {
		return FALSE;
	}
	if (c_th != -1) {
		return TRUE;
	}
	// zXg擾
	if (serveraddr == 0) {
		save_log(TEXT("gethostbyname"), server, LOG_LEVEL_STANDARD);
		serveraddr = inet_addr(server);
		if (serveraddr == -1) {
			serverhostent = gethostbyname(server);
			if (serverhostent == NULL) {
				save_error(TEXT("gethostbyname"), WSAGetLastError());
				return FALSE;
			} else {
				serveraddr = *((unsigned long *)((serverhostent->h_addr_list)[0]));
			}
		}
	}
	if ((c_th = _beginthread(client_send_thread, 0, NULL)) == -1) {
		save_error(TEXT("_beginthread"), GetLastError());
		return FALSE;
	}
	return TRUE;
}
/* End of source */
